use std::sync::LazyLock;

use mlua::Lua;

use crate::AResult;

pub static LUA_CPATH: LazyLock<String> = LazyLock::new(|| {
    String::from_utf8_lossy(
        &std::process::Command::new("luarocks")
            .arg("path")
            .arg("--lr-cpath")
            .arg("--full")
            .output()
            .map_err::<std::vec::Vec<u8>, _>(|_| vec![])
            .map(|r| r.stdout)
            .unwrap_or_default(),
    )
    .trim()
    .to_string()
});

pub static LUA_PATH: LazyLock<String> = LazyLock::new(|| {
    String::from_utf8_lossy(
        &std::process::Command::new("luarocks")
            .arg("path")
            .arg("--lr-path")
            .arg("--full")
            .output()
            .map_err::<std::vec::Vec<u8>, _>(|_| vec![])
            .map(|r| r.stdout)
            .unwrap_or_default(),
    )
    .trim()
    .to_string()
});

pub struct LuaHandler {
    lua: Lua,
}

impl LuaHandler {
    pub fn new() -> AResult<Self> {
        let lua = unsafe { Lua::unsafe_new() };

        let this = Self { lua };

        this.exec(&format!(
            "package.path = \"{}\"\npackage.cpath = \"{}\"",
            &*LUA_PATH, &*LUA_CPATH
        ))?;

        Ok(this)
    }

    pub fn exec(&self, blob: &str) -> AResult<()> {
        self.lua.load(blob).exec()?;
        Ok(())
    }
}
